<?php

if (file_exists(dirname(__FILE__) . '/../../tools/pclzip/pclzip.lib.php')) {
    require_once(dirname(__FILE__) . '/../../tools/pclzip/pclzip.lib.php');
} else {
    require_once(dirname(__FILE__).'/Export/pclzip/pclzip.lib.php');
}

include_once('themler.php');
include_once(dirname(__FILE__).'/Export/Chunk.php');
include_once(dirname(__FILE__).'/Export/FormRendererHelper.php');
include_once(dirname(__FILE__).'/Export/HookController.php');
include_once(dirname(__FILE__).'/Export/LogFormatter.php');
include_once(dirname(__FILE__).'/Export/Package.php');
include_once(dirname(__FILE__).'/Export/PermissionsException.php');
include_once(dirname(__FILE__).'/Export/PreviewHelper.php');
include_once(dirname(__FILE__).'/Export/TemplateBuilder.php');
include_once(dirname(__FILE__).'/Export/TemplateHelper.php');
include_once(dirname(__FILE__).'/Export/ThemeInstallHelper.php');
include_once(dirname(__FILE__).'/Export/Translations.php');
include_once(dirname(__FILE__).'/Export/Version.php');

//set_error_handler('errorHandler', E_ALL); // Deprecated

class ThemeEditor
{
    private $_themeName;
    private $_previewThemeName;
    private $_themeDir;
    private $_previewThemeDir;
    private $_modulesDir;
    private $_templateBuilder;

    public function __construct($themeName)
    {
        $this->_themeName = $themeName;
        $this->_previewThemeName = $themeName . _PREVIEW_SUFFIX_;
        $this->_themeDir = FilesHelper::normalizePath(getThemeDir($this->_themeName));
        $this->_previewThemeDir = FilesHelper::normalizePath(getThemeDir($this->_previewThemeName));
        $this->_modulesDir = FilesHelper::normalizePath(_PS_MODULE_DIR_);
        $this->_templateBuilder = new TemplateBuilder($this->_themeDir);
    }

    /**
     * Saves Themler project to current theme
     */
    public function save($info)
    {
        $chunk = new Chunk();

        if (!$chunk->save($info)) {
            header($_SERVER['SERVER_PROTOCOL'] . ' 400 Bad Request', true, 400);
            return array();
        }

        if ($chunk->last())
        {
            ProviderLog::start('save');
            ProviderLog::start('save:json decode');
            // merge and decode content
            $result = $chunk->complete();
            if ($result['status'] === 'done') {
                $result = json_decode($result['data'], true);
            } else {
                ProviderLog::end('save:json decode');
                ProviderLog::end('save');
                $result['log'] = ProviderLog::getLog();
                $result['result'] = 'error';
                return $result;
            }

            ProviderLog::end('save:json decode');

            $thumbnails = array_key_exists('thumbnails', $result) > 0 ? $result['thumbnails'] : '';
            $cssJsSources = array_key_exists('cssJsSources', $result) > 0 ? $result['cssJsSources'] : '';
            $md5Hashes = array_key_exists('md5Hashes', $result) > 0 ? $result['md5Hashes'] : '';
            $projectData = array_key_exists('projectData', $result) > 0 ? $result['projectData'] : '';
            $images = empty($result['images']) ? null : $result['images'];

            $helper = new PreviewHelper($this->_themeDir);
            $changed_files = getPreviewChangedFiles($this->_themeDir);

            FilesHelper::deleteFile($this->_themeDir . '/assets/css/style.min.css');
            FilesHelper::deleteFile($this->_themeDir . '/assets/css/bootstrap.min.css');
            FilesHelper::createDir($this->_themeDir . '/assets/img');

            ProviderLog::start('Save files');
            foreach ($changed_files as $file) {
                $path = $this->_previewThemeDir . $file;
                $themeFileDir = dirname($path);
                if (file_exists($path)) {
                    if (strpos($file, 'themler/')) {
                        // rewrite module files
                        FilesHelper::copyRecursive($path, $this->_modulesDir . $file);
                    }

                    if (preg_match('#[\\\/]img$#', $themeFileDir)) { // file inside 'img' directory
                        FilesHelper::renameFile($path, $this->_themeDir . $file);
                    } else {
                        FilesHelper::copyRecursive($path, $this->_themeDir . $file, array($helper, "removeDataId"));
                    }
                } else {
                    $helper->removeKey($file);
                    FilesHelper::deleteFile($this->_themeDir . $file);
                }
            }
            ProviderLog::end('Save files');

            ProviderLog::start('Save images');
            $this->_processImages($this->_themeDir, $images, $changed_files);
            ProviderLog::end('Save images');

            ProviderLog::start('Update module files');
            // backward fix with module override files
            FilesHelper::copyRecursive($this->_modulesDir . '/themler/override', FilesHelper::normalizePath(_PS_OVERRIDE_DIR_));

            ProviderLog::end('Update module files');
            setPreviewChangedFiles($this->_themeDir, $this->_previewThemeDir, array());
            $helper->save();

            $this->_saveThumbnails($thumbnails);
            if (!empty($projectData))
                saveProject($this->_themeName, $projectData);

            if (!empty($cssJsSources))
                setThemeCache($this->_themeDir, $cssJsSources);

            if (!empty($md5Hashes))
                setThemeHashes($this->_themeDir, $md5Hashes);

            ProviderLog::start('updateTranslations');
            updateTranslations($this->_themeDir);
            ProviderLog::end('updateTranslations');

            ProviderLog::start('_fixEmptyLayoutFiles');
            $this->_fixEmptyLayoutFiles();
            ProviderLog::end('_fixEmptyLayoutFiles');

            $this->_fixThemeName($this->_themeDir);

            // update themler module files
            FilesHelper::emptyDirRecursive($this->_modulesDir . '/themler');
            FilesHelper::copyRecursive($this->_themeDir . '/dependencies/modules/themler', $this->_modulesDir . '/themler');

            ProviderLog::end('save');

            return array('result' => 'done', 'log' => ProviderLog::getLog());
        }
        return array('result' => 'processed');
    }

    /**
     * Exports Themler project
     */
    public function export($info)
    {
        $chunk = new Chunk();

        if (!$chunk->save($info)) {
            header($_SERVER['SERVER_PROTOCOL'] . ' 400 Bad Request', true, 400);
            return array();
        }

        if ($chunk->last())
        {
            ProviderLog::start('export');
            ProviderLog::start('export:json_decode');
            $result = $chunk->complete();
            if ($result['status'] === 'done') {
                $content = json_decode($result['data'], true);
            } else {
                ProviderLog::end('export:json_decode');
                ProviderLog::end('export');
                $result['log'] = ProviderLog::getLog();
                $result['result'] = 'error';
                return $result;
            }

            ProviderLog::end('export:json_decode');
            $this->_exportTheme($content);
            ProviderLog::end('export');
            return array('result' => 'done', 'log' => ProviderLog::getLog());
        }
        return array('result' => 'processed');
    }

    /**
     * Updates Billion Themler theme
     */
    public function update()
    {
        ProviderLog::start('update');
        $helper = new PreviewHelper($this->_themeDir);
        $changedFiles = getPreviewChangedFiles($this->_themeDir);

        // restore dependencies folder broken after installation
        if (!file_exists($this->_themeDir . '/dependencies/modules/themler')) {
            FilesHelper::createDir($this->_themeDir . '/dependencies');
            FilesHelper::createDir($this->_themeDir . '/dependencies/modules');
            FilesHelper::copyRecursive($this->_modulesDir . '/themler', $this->_themeDir . '/dependencies/modules/themler');
        }

        if (!file_exists($this->_previewThemeDir)) {
            // full copy
            FilesHelper::copyRecursive($this->_themeDir, $this->_previewThemeDir, array($helper, "restoreDataId"));
            $project_dir = $this->_previewThemeDir . '/export';
            if (is_dir($project_dir)) {
                FilesHelper::removeDir($project_dir);
            }
        } else {
            // copy only changed files
            foreach ($changedFiles as $file){
                if (file_exists($this->_themeDir . $file)) {
                    FilesHelper::copyRecursive($this->_themeDir . $file, $this->_previewThemeDir . $file, array($helper, "restoreDataId"));
                } else {
                    FilesHelper::deleteFile($this->_previewThemeDir . $file);
                }
            }
            setPreviewChangedFiles($this->_themeDir, $this->_previewThemeDir, array());
        }

        $this->_fixThemeName($this->_previewThemeDir);

        ProviderLog::end('update');
        return array('result' => 'done', 'log' => ProviderLog::getLog());
    }

    public function remove($themeName)
    {
        $themeDir = FilesHelper::normalizePath(getThemeDir($themeName));
        if (!is_dir($themeDir))
            return array('errors' => 'Theme `' . $themeName .'` does not exist');
        if (Themler::isActiveTheme($themeName))
            return array('errors' => 'Theme `' . $themeName .'` is active');

        ProviderLog::start('removeTheme');

        $helper = new ThemeInstallHelper();
        $previewName = $themeName . _PREVIEW_SUFFIX_;
        $dirs = array(
                    $themeName => $themeDir,
                    $previewName => FilesHelper::normalizePath(getThemeDir($previewName))
                );

        foreach ($dirs as $name => $dir) {
            if ($helper->isThemeInstalled($name))
                Themler::deleteTheme($name);
            if (is_dir($dir))
                FilesHelper::removeDir($dir);
        }

        ProviderLog::end('removeTheme');
        return array('result' => 'done', 'log' => ProviderLog::getLog());
    }

    public function activate($newName = '')
    {
        $themeName = $newName ? $newName : $this->_themeName;
        $themeDir = FilesHelper::normalizePath(getThemeDir($themeName));
        if (!is_dir($themeDir))
            return array('errors' => 'Theme `' . $themeName .'` doesn\'t exist');

        ProviderLog::start('activate');
        $result = Themler::setActiveThemeForShop($themeName);
        ProviderLog::end('activate');
        return array('result' => 'done', 'log' => ProviderLog::getLog());
    }

    public function getFiles($mask, $filter)
    {
        $files = $this->_getFiles($this->_themeDir . '/assets/css/{' . $mask . '}', GLOB_BRACE);

        $out_files = array();
        foreach ($files as $file) {
            $info = pathinfo($file);
            $filename = $info['basename'];
            if (is_dir($file) || !preg_match("#bootstrap\.css|style\.css#", $filename) || ($filter && preg_match("#$filter#", $filename))) {
                continue;
            }
            $out_files[$filename] = FilesHelper::readFile($file);
        }

        return array('files' => $out_files);
    }

    public function setFiles($files)
    {
        if ($files && count($files)) {
            foreach ($files as $filename => $content) {
                $fileExt = pathinfo($filename, PATHINFO_EXTENSION);
                if (in_array($fileExt, array('css'))) {
                    FilesHelper::writeFile($this->_themeDir . '/assets/css/' . $filename, $content, LOCK_EX);
                } elseif ($filename === '/export/themler.manifest') {
                    theme_save_manifest($this->_themeName, $content);
                } else {
                    FilesHelper::writeFile($this->_themeDir . '/' . $filename, $content, LOCK_EX);
                }
            }
        }

        return;
    }

    public function zipFso($data)
    {
        $base_upload_dir = get_base_upload_dir();
        $tmp_dir = $base_upload_dir . '/zip-data.tmp';
        $zip_file = $base_upload_dir . '/zip-data.zip';
        FilesHelper::emptyDirRecursive($tmp_dir);

        $changed_files = array();
        $replace_data = array();
        $this->_fillThemeStorage($data['fso'], $changed_files, $replace_data, false, $tmp_dir);

        $archive = new PclZip($zip_file);

        if (0 == $archive->create($tmp_dir, PCLZIP_OPT_REMOVE_PATH, $tmp_dir))
            throw new Exception("Extract error : " . $archive->errorInfo(true));

        $result = base64_encode(FilesHelper::readFile($zip_file));
        FilesHelper::emptyDirRecursive($tmp_dir);
        FilesHelper::deleteFile($zip_file);
        return $result;
    }

    public function unZip()
    {
        ProviderLog::start('unZip');
        $result = upload($this->_themeName, 'unZip');
        ProviderLog::end('unZip');
        return $result;
    }

    private function prepareCopyRename($newThemeName, $rename = true)
    {
        moveInnerPreview($this->_themeName); // inner preview can exist if theme was upload from Themler in zip archive
        $newThemeDir = FilesHelper::normalizePath(getThemeDir($newThemeName));
        $newThemePreviewDir = FilesHelper::normalizePath(getThemeDir($newThemeName . _PREVIEW_SUFFIX_));

        processRename($this->_themeDir, $this->_previewThemeDir, $newThemeDir, $newThemePreviewDir, $newThemeName, $rename);
    }

    public function copy($new_theme_name = '')
    {
        if (!is_dir($this->_themeDir))
            return array('errors' => 'Theme `' . $this->_themeName .'` doesn\'t exist');

        ProviderLog::start('copyTheme');

        $new_theme_name = $new_theme_name ? checkThemeName($new_theme_name) : checkThemeName($this->_themeName);
        $new_theme_dir = dirname($this->_themeDir) . '/' . $new_theme_name;
        FilesHelper::copyRecursive($this->_themeDir, $new_theme_dir);
        $this->_fixThemeName($new_theme_dir);

        ProviderLog::end('copyTheme');
        return array('result' => 'done', 'log' => ProviderLog::getLog(), 'newName' => $new_theme_name);
    }

    public function rename($newThemeName)
    {
        $result = $this->canRename($newThemeName);
        if ($result['message'])
            return $result;

        if (!is_dir($this->_themeDir))
            return array('errors' => 'Theme `' . $this->_themeName .'` doesn\'t exist');

        ProviderLog::start('rename');

        $isActive = Themler::isActiveTheme($this->_themeName);
        $this->prepareCopyRename($newThemeName);

        Themler::deleteTheme($this->_themeName);
        Themler::deleteTheme($this->_previewThemeName);
        Themler::addTheme($newThemeName);
        if ($isActive) // renamed theme was activated before
            Themler::setActiveThemeForShop($newThemeName);

        ProviderLog::end('rename');
        return array('result' => 'done', 'log' => ProviderLog::getLog());
    }

    public function canRename($newThemeName)
    {
        $result = $this->validate($newThemeName);
        if ($result['message'])
            return $result;

        if (file_exists(FilesHelper::normalizePath(getThemeDir($newThemeName))))
            $result['message'] = 'Theme with such name already exists';

        return $result;
    }

    public function validate($newThemeName)
    {
        $msg = $newThemeName ? '' : 'Theme name cannot be empty';
        if (!Validate::isDirName($newThemeName))
            $msg = 'Not valid theme name ' . $newThemeName;
        return array('result' => 'done', 'message' => $msg);
    }

    /**
     * Returns theme to Billion Themler
     */
    public function getTheme($userThemeName, $includeThemler)
    {
        ProviderLog::start('getTheme');
        moveInnerPreview($this->_themeName);
        $archive = generateArchive($this->_themeName, $userThemeName, $includeThemler);
        ProviderLog::end('getTheme');
        if (is_array($archive) && array_key_exists('content', $archive))
            return $archive;
        trigger_error('Archive is incorrect: "' . $archive . '"', E_USER_ERROR);
    }

    /**
     * Exports Themler project
     */
    private function _exportTheme($content)
    {
        $changed_files = getPreviewChangedFiles($this->_themeDir);

        $theme          = isset($content['themeFso'])     ? $content['themeFso']     : null;
        $images         = isset($content['images'])       ? $content['images']       : null;
        $fonts          = isset($content['iconSetFiles']) ? $content['iconSetFiles'] : null;

        $replaceInfo = array();
        $replaceInfo = array_merge_recursive($replaceInfo, (array)$this->_processFonts($this->_previewThemeDir, $fonts, $changed_files));
        $replaceInfo = array_merge_recursive($replaceInfo, (array)$this->_processImages($this->_previewThemeDir, $images, $changed_files));

        ProviderLog::start('_fillThemeStorage');
        $this->_fillThemeStorage($theme, $changed_files, $replaceInfo, $this->_previewThemeDir);
        ProviderLog::end('_fillThemeStorage');

        ProviderLog::start('_buildThemeLayoutFilesFromHtml'); // backward for custom templates
        $this->_buildThemeLayoutFilesFromHtml($changed_files, $replaceInfo);
        ProviderLog::end('_buildThemeLayoutFilesFromHtml');

        $this->_fixThemeName($this->_previewThemeDir);

        setPreviewChangedFiles($this->_themeDir, $this->_previewThemeDir, $changed_files);
    }

    private function _fixThemeName($theme_dir) {
        $theme_name = basename($theme_dir);
        // TODO: add to changed files (?)
        $config_content = file_get_contents($theme_dir . '/config/theme.yml');
        $config_content = preg_replace('#^name:\s?.*$#m', 'name: ' . $theme_name, $config_content);
        $config_content = preg_replace('#^display_name:\s?.*$#m', 'display_name: ' . $theme_name, $config_content);
        file_put_contents($theme_dir . '/config/theme.yml', $config_content);

        $config_content = file_get_contents($theme_dir . '/config.xml');
        $config_content = preg_replace('#<theme_key>([^<]*)<#m', "<theme_key>$theme_name<", $config_content);
        file_put_contents($theme_dir . '/config.xml', $config_content);
    }

    private function _processImages($themeDir, $images, &$changed_files)
    {
        if (!isset($images) || !is_array($images)) return null;

        ProviderLog::start('_processImages');
        $imageDir = $themeDir . '/assets/img';
        FilesHelper::createDir($imageDir);

        $ids = array();
        $paths = array();
        foreach ($images as $id => $content)
        {
            /* STORAGE */
            $imageFileName = preg_replace('/[^a-z0-9_\.]/i', '', $id);
            $imagePath = $imageDir . '/' . $imageFileName;

            if ($content) {
                $changed_files[] = $imagePath;
                if ($content === '[DELETED]') {
                    FilesHelper::deleteFile($imagePath);
                    continue;
                }
                FilesHelper::writeFile($imagePath, base64_decode($content));
            }

            $ids[] = $id;
            $paths[] = IMG_PATH_TOKEN . '/' . $imageFileName; // css path
        }

        ProviderLog::end('_processImages');
        return array('ids' => $ids, 'paths' => $paths);
    }

    /**
     * Saves theme fonts
     */
    private function _processFonts($themeDir, $fonts, &$changed_files)
    {
        if (!isset($fonts)) {
            return null;
        }

        ProviderLog::start('_processFonts');
        $fonts_dir = $themeDir . '/assets/fonts';
        FilesHelper::createDir($fonts_dir);

        foreach ($fonts as $name => $content) {
            $path = $fonts_dir . '/' . $name;
            FilesHelper::writeFile($path, base64_decode($content));
            $changed_files[] = $path;
        }

        $existing_fonts = scandir($fonts_dir);
        $ids = array();
        $paths = array();
        foreach ($existing_fonts as $filename) {
            if ($filename !== '.' && $filename !== '..') {
                $ids[] = $filename;
                $paths[] = '../fonts/' . $filename;
            }
        }

        ProviderLog::end('_processFonts');
        return array('ids' => $ids, 'paths' => $paths);
    }

    private function _saveThumbnails($thumbnails)
    {
        if (is_array($thumbnails)) {
            ProviderLog::start('_saveThumbnails');
            foreach ($thumbnails as $thumbnail) {
                $name = $thumbnail['name'];
                $img = base64_decode(str_replace('data:image/png;base64,', '', $thumbnail['data']));

                FilesHelper::writeFile($this->_themeDir . '/' . $name, $img);
                FilesHelper::writeFile($this->_previewThemeDir . '/' . $name, $img);
            }
            ProviderLog::end('_saveThumbnails');
        }
    }

    private function _fillThemeStorage($fso, &$changed_files, &$replaceInfo, $root, $inner_folder_path = '')
    {
        if (!isset($fso) || !is_array($fso) || !is_array($fso['items'])) {
            return;
        }

        foreach ($fso['items'] as $name => $file) {
            if (isset($file['content']) && isset($file['type'])) {
                $content = $file['type'] == 'text' ? $file['content'] : base64_decode($file['content']);
                $this->_processFile($root, $inner_folder_path, $name, $content, $changed_files, $replaceInfo);
            } elseif (isset($file['items']) && isset($file['type'])) {
                $to = empty($inner_folder_path) ? $name : $inner_folder_path . '/' . $name;
                $this->_fillThemeStorage($file, $changed_files, $replaceInfo, $root, $to);
            }
        }
    }

    private function _processFile($root, $inner_folder_path, $file_name, $content, &$changed_files, &$replaceInfo)
    {
        $folders = correct_version_path($inner_folder_path);
        foreach ($folders as $folder) {
            $path = FilesHelper::normalizePath($root . '/' . $folder);
            FilesHelper::createDir($path);

            $path .= '/' . $file_name;
            $info = pathinfo($path);
            $fileName = $info['basename'];
            $fileExt = isset($info['extension']) && $info['extension'] ? $info['extension'] : '';

            if ($content === '[DELETED]') {
                FilesHelper::deleteFile($path);
            } else if (in_array($fileExt, array('css')) && !strpos($path, 'themler/')) {
                if (dirname($path) === $this->_previewThemeDir) {
                    $path = $root . '/assets/css/' . $fileName;
                }
                if (!empty($replaceInfo['ids']) && !empty($replaceInfo['paths'])) {
                    $content = str_replace($replaceInfo['ids'], $replaceInfo['paths'], $content);
                }
                FilesHelper::writeFile($path, $content);
            } else if (in_array($fileExt, array('js')) && !strpos($path, 'themler/')) {
                if (dirname($path) === $this->_previewThemeDir) {
                    $path = $root . '/assets/js/' . $fileName;
                }
                FilesHelper::writeFile($path, $content);
            } else if (in_array($fileExt, array('tpl'))) {
                $result = $this->_replaceImageIdToUrl($content, $replaceInfo);
                FilesHelper::writeFile($path, $result);
            } else { // other files
                FilesHelper::writeFile($path, $content);
            }
            $changed_files[] = $path; // save changed file - source or modified
        }
    }

    /**
     * Build theme layout files header.tpl, footer.tpl
     */
    private function _buildThemeLayoutFilesFromHtml(&$changed_files, $replaceInfo)
    {
        $dir = $this->_previewThemeDir . '/templates';
        foreach (FilesHelper::enumerateFiles($dir) as $file)
        {
            $path = $file['path'];
            $info = pathinfo($path);
            $fileExt = isset($info['extension']) && $info['extension'] ? $info['extension'] : '';
            if (!in_array($fileExt, array('html'))) {
                continue;
            }

            $content = $this->_replaceImageIdToUrl($file['content'], $replaceInfo);
            $changes = $this->_templateBuilder->build($info, $content);
            $changed_files = array_merge($changed_files, $changes);
            FilesHelper::deleteFile($path);
            $changed_files[] = $path;
        }
    }

    private function _fixEmptyLayoutFiles()
    {
        $type = 'default';
        $dirs = array($this->_previewThemeDir, $this->_themeDir);
        foreach ($dirs as $dir)
        {
            foreach (FilesHelper::enumerateFiles($dir . '/templates') as $file)
                $this->_addCommentToEmptyFile($file['path'], $file['content']);

            $path404 = $dir . '/footer_template404.tpl';
            if (file_exists($path404))
                $this->_addCommentToEmptyFile($path404, FilesHelper::readFile($path404));

            $path = $dir . '/templates/templates.php';
            if (file_exists($path)) {
                require_once($path);
            }
            if (isset($templates) && isset($templates[$type])) {
                foreach ($templates[$type] as $t) {
                    if (isset($t['name']) && ($t['name'] == $type)) {
                        $value = $type . '_' . $t['id'];
                        $header = $dir . '/templates/header_' . $value . '.tpl';
                        $footer = $dir . '/templates/footer_' . $value . '.tpl';
                        if (file_exists($header) && file_exists($footer)) {
                            $this->_templateBuilder->buildLayoutFiles($dir, '',
                                '<!--Backward-->'.FilesHelper::readFile($header),
                                FilesHelper::readFile($footer)
                            );
                            break;
                        }
                    }
                }
            }
        }
    }

    private function _addCommentToEmptyFile($path, $content)
    {
        if (empty($content))
            FilesHelper::writeFile($path, '<!--Empty file content-->');
        return;
    }

    private function _replaceImageIdToUrl($content, $replaceInfo)
    {
        $result = $content;
        if (!empty($replaceInfo['ids']) && !empty($replaceInfo['paths'])) {
            foreach ($replaceInfo['ids'] as $key => $value) { // replace <img src="url(id)"> to <img src="$url">
                $path = $this->_themeGetImagePath($replaceInfo['paths'][$key]);
                $result = str_replace('url(' . $value . ')', $path, $result);
            }
        }
        $result = preg_replace('#src=["\']url\((https?://[^\)]+)\)["\']#', 'src="$1"', $result);

        return $result;
    }

    private function _themeGetImagePath($image)
    {
        $output = <<<EOT
{assign var=theme_name value=\$tpl_dir|regex_replace:'/(.*)([\/\\\\\]themes[\/\\\\\])(.*)(_preview[\/\\\\\]$)/':'$3'}
{if \$isBdPreview && !file_exists("\$tpl_dir/assets/img/$image")}{\$img_dir}../../../{\$theme_name}/assets/img/$image{else}{\$img_dir}$image{/if}
EOT;
        return $output;
    }

    public function chunkClear()
    {
        $chunk = new Chunk();
        FilesHelper::removeDir($chunk->UPLOAD_PATH);
    }

    public function uploadFile()
    {
        ProviderLog::start('uploadFile');
        $result = upload($this->_themeName, 'uploadFile');
        ProviderLog::end('uploadFile');
        return $result;
    }

    public function uploadTheme()
    {
        ProviderLog::start('uploadTheme');
        $result = upload($this->_themeName, 'uploadTheme');
        ProviderLog::end('uploadTheme');
        return $result;
    }

    private function _getFiles($mask, $flags)
    {
        $files = glob($mask, $flags);
        if (!is_array($files)) {
            $files = array();
        }
        $folders = glob(dirname($mask) . '/*', GLOB_ONLYDIR | GLOB_NOSORT);
        if (!is_array($folders)) {
            $folders = array();
        }
        foreach ($folders as $dir) {
            $files = array_merge($files, $this->_getFiles($dir . '/' . basename($mask), $flags));
        }

        return $files;
    }
}