<?php

use PrestaShop\PrestaShop\Adapter\ObjectPresenter;

class FrontControllerOverrideHelper
{
    public static function init($front_controller, $context)
    {
        if (Tools::getIsset('theme_name'))
            self::registerPreviewErrorHandler();


        $css_dir = _THEME_CSS_DIR_;
        $img_dir = _THEME_IMG_DIR_;
        $js_dir = _THEME_JS_DIR_;

        $context->smarty->assign(array_merge(array(
            'tpl_dir' => getThemeDir(),
            'img_dir' => $img_dir,
            'css_dir' => $css_dir,
            'js_dir' => $js_dir,
            'css_theme_cache' => Configuration::get('PS_CSS_THEME_CACHE'),
            'bd_order_process' => Configuration::get('PS_ORDER_PROCESS_TYPE') ? 'order-opc' : 'order',
            'slider_vars' => getSliderModeVars(),
            'template_vars' => getTemplateVars(),
        ), self::getLanguageVariables($context), self::getCurrencyVariables($context)));
    }


    public static function getLanguageVariables($context)
    {
        $languages = Language::getLanguages(true, $context->shop->id);

        foreach ($languages as &$lang) {
            $lang['name_simple'] = self::getNameSimple($lang['name']);
        }

        return array(
            'languages' => $languages,
            'current_language' => array(
                'id_lang' => $context->language->id,
                'name' => $context->language->name,
                'iso_code' => $context->language->iso_code,
                'name_simple' => self::getNameSimple($context->language->name)
            )
        );
    }

    public static function getCurrencyVariables($context)
    {
        $current_currency = null;
        $serializer = new ObjectPresenter;
        $currencies = array_map(
            function ($currency) use ($context, $serializer, &$current_currency) {
                $currencyArray = $serializer->present($currency);

                // serializer doesn't see 'sign' because it is not a regular
                // ObjectModel field.
                $currencyArray['sign'] = $currency->sign;

                $url = $context->link->getLanguageLink($context->language->id);

                $extraParams = array(
                    'SubmitCurrency' => 1,
                    'id_currency' => $currency->id
                );

                $partialQueryString = http_build_query($extraParams);
                $separator = empty(parse_url($url)['query']) ? '?' : '&';

                $url .= $separator . $partialQueryString;

                $currencyArray['url'] = $url;

                if ($currency->id === $context->currency->id) {
                    $currencyArray['current'] = true;
                    $current_currency = $currencyArray;
                } else {
                    $currencyArray['current'] = false;
                }

                return $currencyArray;
            },
            Currency::getCurrencies(true, true)
        );

        return array(
            'currencies' => $currencies,
            'current_currency' => $current_currency
        );
    }

    private static function getNameSimple($name)
    {
        return preg_replace('/\s\(.*\)$/', '', $name);
    }

    private static function registerPreviewErrorHandler()
    {
        register_shutdown_function(function () {
            if ($e = error_get_last()) {
                switch ($e['type']) {
                    case E_ERROR:
                    case E_CORE_ERROR:
                    case E_COMPILE_ERROR:
                    case E_PARSE:
                        printf('[PHP_ERROR]%s[PHP_ERROR]', json_encode($e));
                }
            }
        });
    }
}